SmolBitSet

Struct SmolBitSet 

pub struct SmolBitSet { /* private fields */ }
A dynamically sized bitset with memory usage optimizations.

Implementations

Source

impl SmolBitSet

pub fn and_not(self, rhs: &Self) -> Self

Unsets all bits set on the rhs SmolBitSet.

This is equivalent to self & !rhs with integers.

pub fn and_not_assign(&mut self, rhs: &Self)

Unsets all bits set on the rhs SmolBitSet.

This is equivalent to *self &= !rhs with integers.

Source

impl SmolBitSet

pub const fn new() -> Self

Constructs a new, empty SmolBitSet.

Examples
let mut sbs = SmolBitSet::new();
pub const fn new_small(val: usize) -> Self

Constructs a new SmolBitSet from the provided val without any heap allocations.

Panics

Panics if any of the 2 most significant bits in val is 1.

Examples
const sbs: SmolBitSet = SmolBitSet::new_small(1234);
assert_eq!(sbs, SmolBitSet::from(1234u16));
pub const fn new_flag(bit: u32) -> Self

Constructs a new sparse SmolBitSet from the provided bit index without any heap allocations.

Panics

Panics if bit is larger than 2^30.

Examples
const sbs: SmolBitSet = SmolBitSet::new_flag(1234);
assert_eq!(sbs, SmolBitSet::new_flag(0) << 1234);
pub const fn from_bits_small<const N: usize>(bits: [usize; N]) -> Self

Constructs a new SmolBitSet from the provided array of bit indices without any heap allocations.

Panics

Panics if any bit index in bits is larger than or equal to usize::BITS - 2.

Examples
const sbs: SmolBitSet = SmolBitSet::from_bits_small([0, 4, 1, 6]);
assert_eq!(sbs, SmolBitSet::from(0b0101_0011u8));
// this panics since 62 is outside of the range
// a SmolBitSet can hold without incurring a heap allocation
let sbs = SmolBitSet::from_bits_small([62]);
// this fails to compile since the const evaluation
// panics for the same reason as above
const sbs: SmolBitSet = SmolBitSet::from_bits_small([62]);
pub fn from_bits(bits: &[usize]) -> Self

Creates a new SmolBitSet from the provided slice of bit indices.

Examples
// bit indices can be in any order
let sbs = SmolBitSet::from_bits(&[0, 6, 4, 3]);
assert_eq!(sbs, SmolBitSet::from(0b0101_1001u8));

let sbs = SmolBitSet::from_bits(&[63]);
assert_eq!(sbs, SmolBitSet::from(1u64 << 63));